package p12f.exe.holdercert.utils;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;

import org.bouncycastle.util.encoders.Hex;

import p12f.exe.pasarelapagos.exceptions.ConfigLoadException;
import p12f.exe.pasarelapagos.objects.AditionalData;
import p12f.exe.pasarelapagos.objects.HolderCertCertifiedData;
import p12f.exe.pasarelapagos.objects.HolderCertCertifiedDataList;
import p12f.exe.pasarelapagos.objects.HolderCertData;
import p12f.exe.pasarelapagos.objects.HolderCertDataList;
import p12f.exe.pasarelapagos.objects.HolderCertResponse;
import p12f.exe.pasarelapagos.objects.config.FinantialOrgConfig;
import p12f.exe.pasarelapagos.security.crypto.TripleDESEBCDICCryptoHelper;
import p12f.exe.pasarelapagos.utils.CSBHelpper;

import com.ejie.r01f.util.StringUtils;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;

/**
 * Utilidad de ayuda para la clase de validacin de cuentas.
 *
 *      IBAN
 *      ================
 *      El IBAN consta de un mximo de 34 caracteres alfanumricos. 
 *      Los dos primeros son de carcter alfabtico e identifican el pas. 
 *      Los dos siguientes son dgitos de control y son el elemento de validacin de la totalidad del IBAN. 
 *      Los restantes son el nmero de cuenta, que en la mayora de los casos identifica adems la entidad y la oficina.
 *      
 *      En el caso espaol estn los cuatro primeros, formados por los elementos explicados en el prrafo anterior, 
 *      y luego se sigue con los 20 caracteres numricos actuales del Cdigo Cuenta Cliente (es decir, consta de un total de 24 caracteres). 
 *      Dichos cuatro nuevos caracteres corresponden a:
 *      		Los dos primeros, al pas (ES para Espaa). 
 *      		Los dos siguientes, a los dgitos de control IBAN. 
 *      
 *      Ejemplos: comparativa entre el cdigo cuenta cliente (CCC) y cdigo IBAN de una misma cuenta.
 *      		Cdigo Cuenta Cliente (C.C.C.): 2077 0024 00 3102575766
 *      		Cdigo IBAN: IBAN ES76 2077 0024 0031 0257 5766 (formato papel).
 *      		Cdigo IBAN: ES7620770024003102575766 (formato electrnico).
 */
public class HolderCertUtil {
	
	/**
	 * Genera un oid a partir de los identificadores de administracin, tipo y referencia.
	 * @param admin Identificador de la administracin asociada al oid
	 * @param type Identificador del tipo asociado al oid
	 * @param reference Referencia asociada al oid
	 * @return String
	 */
	public static String generateOID(String admin, String type, String reference) {
		SimpleDateFormat sd = new SimpleDateFormat("ddMMyy");	
		String identificacion = sd.format(new Date(System.currentTimeMillis()));
		String referenceWith00 = reference + "00";
		String dc = CSBHelpper.getDigitoControlCSB507(admin,type, referenceWith00,identificacion, "0"); 
		String oid =  admin + type + reference + dc + identificacion;
		String formattedString = StringUtils.lPadWithChar(oid, '0',19);
		
		return formattedString;
	}
	
	/**
	 * Obtiene la Entidad Financiera a partir de un HolderCertData.
	 * @param holderCertData Datos a partir de los cuales se obtendr la entidad financiera
	 * @return String
	 * @throws Exception
	 */
	public static String getFinantialOrgCode(HolderCertData holderCertData) throws Exception {
		String itemToCert = holderCertData.itemToCertNumber;
		if (holderCertData.type.equalsIgnoreCase(HolderCertData.TYPE_CREDITCARDATA)) {
			return null;
		} else {
			try {
				if (holderCertData.accountNumberFormat.equalsIgnoreCase(HolderCertData.ACCOUNT_NUMBER_FORMAT_CC)){
					return  itemToCert.substring(0, 4);
				} else if (holderCertData.accountNumberFormat.equalsIgnoreCase(HolderCertData.ACCOUNT_NUMBER_FORMAT_IBAN)){
					return  itemToCert.substring(4, 8);
				}
			} catch (Exception ex) {
				throw new Exception("Error en el formato de cuenta :, pe. Cdigo IBAN: ES7620770024003102575766 (formato electrnico).\n \n  :"+ ex.getLocalizedMessage() + "@ itemToCert=" + itemToCert);
			}
		}
		return null;
	}
	
	/**
	 * Mtodo que comprueba los resultados de un lote certificado para resolver si es necesario
	 * o no enviar un evento de resultado.
	 * @param certifiedDataList
	 * @return
	 */
	public static boolean hasToSendEvent(HolderCertCertifiedDataList certifiedDataList) {
		Iterator it = certifiedDataList.certCertifiedDataList.iterator();
		while (it.hasNext()) {
			HolderCertCertifiedData certifiedData = (HolderCertCertifiedData)it.next();
			if (HolderCertResponse.resultsNoFinal.contains(new Integer(certifiedData.holderCertResponse.certCode))) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Funcin que devuelve los datos de cuenta cifrados segn las claves suministradas.
	 * @param holderCertData Datos de cuenta
	 * @param clave1
	 * @param clave2
	 * @return HolderCertData
	 * @throws Exception
	 */
	public static HolderCertData encrypt (HolderCertData holderCertData, String clave1, String clave2) throws Exception{
		HolderCertData encryptedHolderCertData = new HolderCertData();
		encryptedHolderCertData.accountNumberFormat  = holderCertData.accountNumberFormat;
		encryptedHolderCertData.aditionalDataList    = holderCertData.aditionalDataList;
		encryptedHolderCertData.citizenId            = holderCertData.citizenId;
		encryptedHolderCertData.encType              = holderCertData.encType;
		encryptedHolderCertData.type                 = holderCertData.type;
		encryptedHolderCertData.requestAdminID       = holderCertData.requestAdminID;
		encryptedHolderCertData.requestID            = holderCertData.requestID;
		try {
			if ((clave1 == null) ||  (clave2 == null) || "".equals(clave1) || "".equals(clave2)) {
				throw new Exception ("Las claves suministradas no pueden ser nulas o vacas");
			}
			if (holderCertData.type.equalsIgnoreCase(HolderCertData.TYPE_ACCOUNTNUMBER)) {
				encryptedHolderCertData.itemToCertNumber = _encryptAccountData(clave1,clave2,holderCertData);
			} else {
				if (holderCertData.type.equalsIgnoreCase(HolderCertData.TYPE_CREDITCARDATA)){
					encryptedHolderCertData.itemToCertNumber =_encryptCreditCardData (clave1,clave2,holderCertData);
				}
			}
			return encryptedHolderCertData;
		} catch (XOMarshallerException e) {
			throw new Exception(e);
		} catch (ConfigLoadException e) {
			throw new Exception(e);
		}
	}
	
	/**
	 * Funcin que devuelve los datos de cuenta cifrados segn la entidad financiera a la que correspondan.
	 * @param holderCertData Datos de cuenta
	 * @param entidad Configuracin de la entidad financiera a la que corresponden los datos
	 * @return HolderCertData
	 * @throws Exception
	 */
	public static HolderCertData encrypt (HolderCertData holderCertData, FinantialOrgConfig  entidad) throws Exception{
		String clave1 = (String)entidad.claves.get("accountHolderKey1");
		String clave2 = (String)entidad.claves.get("accountHolderKey2");
		if ((clave1 == null) ||  (clave2 == null) || "".equals(clave1) || "".equals(clave2)) {
			throw new Exception ("No se han definido las claves accountHolderKey1 y  accountHolderKey2 para la Entidad Financiera: " + entidad.oid );
		}
		return encrypt (holderCertData, clave1, clave2);
	}
	
	/**
	 * Funcin que devuelve una lista de datos de cuenta cifrados con las claves suministradas.
	 * @param holderCertDataList Datos de cuentas
	 * @param clave1
	 * @param clave2
	 * @return HolderCertDataList
	 * @throws Exception
	 */
	public static HolderCertDataList encrypt(HolderCertDataList holderCertDataList, String clave1, String clave2) throws Exception {
		HolderCertDataList  encryptedHolderCertDataList  = new HolderCertDataList();
		encryptedHolderCertDataList.packageID = holderCertDataList.packageID;
		for (Iterator it = holderCertDataList.dataList.iterator(); it.hasNext();){
			HolderCertData  holderCertData = (HolderCertData) it.next();
			HolderCertData  encryptedHolderCertData  =  encrypt (holderCertData, clave1, clave2);
			encryptedHolderCertDataList.dataList.add(encryptedHolderCertData);
		}
		encryptedHolderCertDataList.itemCount = holderCertDataList.dataList.size();
		return encryptedHolderCertDataList;
	}
	
	/**
	 * Funcin que devuelve una lista de datos de cuenta cifrados segn la entidad financiera a la que correspondan.
	 * Todas las cuentas de la lista corresponden a la misma entidad.
	 * @param holderCertDataList Datos de cuentas
	 * @param entidad Configuracin de la entidad financiera a la que corresponden los datos
	 * @return HolderCertDataList
	 * @throws Exception
	 */
	public static HolderCertDataList encrypt(HolderCertDataList holderCertDataList,	FinantialOrgConfig  entidad) throws Exception {
		String clave1 = (String)entidad.claves.get("accountHolderKey1");
		String clave2 = (String)entidad.claves.get("accountHolderKey2");
		if ((clave1 == null) ||  (clave2 == null) || "".equals(clave1) || "".equals(clave2)) {
			throw new Exception ("No se han definido las claves accountHolderKey1 y  accountHolderKey2 para la Entidad Financiera: " + entidad.oid );
		}
		return encrypt(holderCertDataList, clave1, clave2);
	}
	
	/**
	 * Funcin que devuelve una cuenta descifrada segn las claves de la entidad financiera suministrada.
	 * @param itemToCertNumber
	 * @param encType
	 * @param entidad
	 * @return
	 * @throws Exception
	 */
	public static String decrypt (String itemToCertNumber, String encType, FinantialOrgConfig  entidad) throws Exception {
		String clave1 = (String)entidad.claves.get("accountHolderKey1");
		String clave2 = (String)entidad.claves.get("accountHolderKey2");
		if ((clave1 == null) ||  (clave2 == null) || "".equals(clave1) || "".equals(clave2)) {
			throw new Exception ("No se han definido las claves accountHolderKey1 y  accountHolderKey2 para la Entidad Financiera: " + entidad.oid );
		}
		return _decryptData(clave1, clave2, itemToCertNumber, encType);
	}
	
	/**
	 * Funcin que devuelve los datos de cuenta descifrados segn las claves suministradas.
	 * @param holderCertData Datos de cuenta
	 * @param clave1
	 * @param clave2
	 * @return HolderCertData
	 * @throws Exception
	 */
	public static HolderCertData decrypt (HolderCertData holderCertData, String clave1, String clave2) throws Exception{
		HolderCertData decryptedHolderCertData = new HolderCertData();
		decryptedHolderCertData.accountNumberFormat  = holderCertData.accountNumberFormat;
		decryptedHolderCertData.aditionalDataList    = holderCertData.aditionalDataList;
		decryptedHolderCertData.citizenId            = holderCertData.citizenId;
		decryptedHolderCertData.encType              = holderCertData.encType;
		decryptedHolderCertData.type                 = holderCertData.type;
		decryptedHolderCertData.requestAdminID       = holderCertData.requestAdminID;
		decryptedHolderCertData.requestID            = holderCertData.requestID;
		try {
			if ((clave1 == null) ||  (clave2 == null) || "".equals(clave1) || "".equals(clave2)) {
				throw new Exception ("Las claves suministradas no pueden ser nulas o vacas");
			}
			decryptedHolderCertData.itemToCertNumber = _decryptData(clave1,clave2,holderCertData).trim();
			return decryptedHolderCertData;
		} catch (XOMarshallerException e) {
			throw new Exception(e);
		} catch (ConfigLoadException e) {
			throw new Exception(e);
		}
	}
	
	/**
	 * Funcin que devuelve los datos de cuenta descifrados segn la entidad financiera a la que correspondan.
	 * @param holderCertData Datos de cuenta
	 * @param entidad Configuracin de la entidad financiera a la que corresponden los datos
	 * @return HolderCertData
	 * @throws Exception
	 */
	public static HolderCertData decrypt (HolderCertData holderCertData, FinantialOrgConfig  entidad) throws Exception{
		String clave1 = (String)entidad.claves.get("accountHolderKey1");
		String clave2 = (String)entidad.claves.get("accountHolderKey2");
		if ((clave1 == null) ||  (clave2 == null) || "".equals(clave1) || "".equals(clave2)) {
			throw new Exception ("No se han definido las claves accountHolderKey1 y  accountHolderKey2 para la Entidad Financiera: " + entidad.oid );
		}
		return decrypt(holderCertData, clave1, clave2);
	}
	
	/**
	 * Funcin que devuelve una lista de datos de cuenta descifrados segn las claves suministradas.
	 * @param holderCertDataList Datos de cuentas
	 * @param clave1
	 * @param clave2
	 * @return HolderCertDataList
	 * @throws Exception
	 */
	public static HolderCertDataList decrypt(HolderCertDataList holderCertDataList, String clave1, String clave2) throws Exception {
		HolderCertDataList  decryptedHolderCertDataList  = new HolderCertDataList();
		decryptedHolderCertDataList.packageID = holderCertDataList.packageID;
		for (Iterator it = holderCertDataList.dataList.iterator(); it.hasNext();){
			HolderCertData  holderCertData = (HolderCertData) it.next();
			HolderCertData  encryptedHolderCertData  =  decrypt (holderCertData, clave1, clave2);
			decryptedHolderCertDataList.dataList.add(encryptedHolderCertData);
		}
		decryptedHolderCertDataList.itemCount = holderCertDataList.dataList.size();
		return decryptedHolderCertDataList;
	}
	
	/**
	 * Funcin que devuelve una lista de datos de cuenta descifrados segn la entidad financiera a la que correspondan.
	 * @param holderCertDataList Datos de cuentas
	 * @param entidad Configuracin de la entidad financiera a la que corresponden los datos
	 * @return HolderCertDataList
	 * @throws Exception
	 */
	public static HolderCertDataList decrypt(HolderCertDataList holderCertDataList, FinantialOrgConfig  entidad) throws Exception {
		String clave1 = (String)entidad.claves.get("accountHolderKey1");
		String clave2 = (String)entidad.claves.get("accountHolderKey2");
		if ((clave1 == null) ||  (clave2 == null) || "".equals(clave1) || "".equals(clave2)) {
			throw new Exception ("No se han definido las claves accountHolderKey1 y  accountHolderKey2 para la Entidad Financiera: " + entidad.oid );
		}
		return decrypt(holderCertDataList, clave1, clave2);
	}
	
	/**
	 * Funcin que devuelve una lista de datos de certificacin descifrados segn las claves suministradas.
	 * @param holderCertCertifiedDataList Datos de certificacin de las cuentas
	 * @param clave1
	 * @param clave2
	 * @return HolderCertCertifiedDataList
	 * @throws Exception
	 */
	public static HolderCertCertifiedDataList decrypt(HolderCertCertifiedDataList holderCertCertifiedDataList, String clave1, String clave2) throws Exception {
		HolderCertCertifiedDataList  decryptedHolderCertDataList  = new HolderCertCertifiedDataList();
		decryptedHolderCertDataList.packageID = holderCertCertifiedDataList.packageID;
		for (Iterator it = holderCertCertifiedDataList.certCertifiedDataList.iterator(); it.hasNext();){
			HolderCertCertifiedData  holderCertCertifiedData = (HolderCertCertifiedData) it.next();
			HolderCertCertifiedData encryptedHolderCertCertifiedData = new HolderCertCertifiedData();
			encryptedHolderCertCertifiedData.holderCertData  =  decrypt (holderCertCertifiedData.holderCertData, clave1, clave2);
			encryptedHolderCertCertifiedData.holderCertResponse = holderCertCertifiedData.holderCertResponse;
			decryptedHolderCertDataList.certCertifiedDataList.add(encryptedHolderCertCertifiedData);
		}
		decryptedHolderCertDataList.itemCount = decryptedHolderCertDataList.certCertifiedDataList.size();
		return decryptedHolderCertDataList;
	}
	
	/**
	 * Funcin que devuelve una lista de datos de certificacin descifrados segn la entidad financiera a la que correspondan.
	 * Todas las cuentas de la lista corresponden a la misma entidad.
	 * @param holderCertCertifiedDataList Datos de certificacin de las cuentas
	 * @param entidad Configuracin de la entidad financiera a la que corresponden los datos
	 * @return HolderCertCertifiedDataList
	 * @throws Exception
	 */
	public static HolderCertCertifiedDataList decrypt(HolderCertCertifiedDataList holderCertCertifiedDataList, FinantialOrgConfig  entidad) throws Exception {
		String clave1 = (String)entidad.claves.get("accountHolderKey1");
		String clave2 = (String)entidad.claves.get("accountHolderKey2");
		if ((clave1 == null) ||  (clave2 == null) || "".equals(clave1) || "".equals(clave2)) {
			throw new Exception ("No se han definido las claves accountHolderKey1 y  accountHolderKey2 para la Entidad Financiera: " + entidad.oid );
		}
		return decrypt(holderCertCertifiedDataList, clave1, clave2);
	}
	
	
	/* ********************************************************************** */
	/* ************************* Metodos Privados *************************** */
	/* ********************************************************************** */
	
	private static String _encryptCreditCardData (String clave1, String clave2 ,HolderCertData holderCertData) throws Exception{
		String creditCardNumber  = holderCertData.itemToCertNumber;		
		String creditCardYear = ((AditionalData)holderCertData.aditionalDataList.get("creditCardYear")).value;
		String creditCardMonth = ((AditionalData)holderCertData.aditionalDataList.get("creditCardMonth")).value;
		
		creditCardNumber = StringUtils.rPadWithChar(creditCardNumber,' ',20);
		String data = creditCardNumber + "20" + creditCardYear + creditCardMonth;
		// 2 Rellenar a 32 los datps finales a encriptar
		data = StringUtils.rPadWithChar(data,' ',32);
		String encType = holderCertData.encType;
		String claveFinal  = new String(Hex.encode(TripleDESEBCDICCryptoHelper.doXOR(clave1, clave2))).toUpperCase();
		TripleDESEBCDICCryptoHelper encrypter = new TripleDESEBCDICCryptoHelper();
		if (HolderCertData.ENC_TYPE_BASE64.equals(encType)){
			return encrypter.encrypt(claveFinal,data); 
		} else { // por defecto cifrado Hexadecimal
			return encrypter.encryptHEX(claveFinal,data);
		}
	}
	
	private static String _encryptAccountData (String clave1, String clave2 ,HolderCertData holderCertData) throws Exception{
		String encType =  holderCertData.encType;
		String data = 	  holderCertData.itemToCertNumber;
		
		if (holderCertData.accountNumberFormat.equalsIgnoreCase(HolderCertData.ACCOUNT_NUMBER_FORMAT_CC)){
			data = StringUtils.rPadWithChar(data,' ',32);
		} else {
			data = StringUtils.rPadWithChar(data,' ',32);
		}
		
		String claveFinal  = new String(Hex.encode(TripleDESEBCDICCryptoHelper.doXOR(clave1, clave2))).toUpperCase();
		TripleDESEBCDICCryptoHelper encrypter = new TripleDESEBCDICCryptoHelper();	
		if (HolderCertData.ENC_TYPE_BASE64.equals(encType)){
			return encrypter.encrypt(claveFinal,data); 
		} else { // por defecto cifrado Hexadecimal
			return encrypter.encryptHEX(claveFinal,data);
		}
	}
	
	private static String _decryptData (String clave1, String clave2, HolderCertData holderCertData) throws Exception {
		String encType =  holderCertData.encType;
		String data = 	  holderCertData.itemToCertNumber;
		return _decryptData(clave1, clave2, data, encType);
	}
	
	private static String _decryptData (String clave1, String clave2, String data, String encType) throws Exception {
		String claveFinal  = new String(Hex.encode(TripleDESEBCDICCryptoHelper.doXOR(clave1, clave2))).toUpperCase();
		TripleDESEBCDICCryptoHelper encrypter = new TripleDESEBCDICCryptoHelper();
		if (HolderCertData.ENC_TYPE_BASE64.equals(encType)){
			return encrypter.decrypt(claveFinal,data).trim(); 
		} else { // por defecto cifrado Hexadecimal
			return encrypter.decryptHEX(claveFinal,data).trim();
		}
	}
	
}
