/*
 * Created on 18-may-2005
 * @author co01347e
 */
package p12f.exe.pasarelapagos.objects;

import java.io.Serializable;
import java.util.Map;

import p12f.exe.pasarelapagos.objects.validation.ValidationConstants;
import p12f.exe.pasarelapagos.utils.ControlCodeHelper;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.NumberUtils;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * Estructura que contiene los datos de un pago.
 */
public class DatosPago implements Serializable, Initializable {

	// Constantes con los tipos de validacion posibles.
	public final static int NO_VALIDAR = 0;
	public final static int VALIDACION_NORMAL = 1;
	public final static int VALIDACION_TRAFICO = 2;
	
	/**
	 * Formato del pago.
	 * 507		Cuadernillo 57
	 * 507-ns	Cuadernillo 57 no estndar
	 * 502		Cuadernillo 60 modalidad 1-corta
	 * 508		Cuadernillo 60 modalidad 1-larga
	 * 521		Cuadernillo 60 modalidad 2 sin recargo
	 * 522		Cuadernillo 60 modalidad 2 con recargo
	 */
	public String formato;
	/**
	 * Indica la forma de validar los dgitos de control:
	 */
	public int validar;
	/**
	 * Cdigo de procedimiento recaudatorio.
	 */
	public String cpr;
	/**
	 * Cdigo completo en el formato 57  60.
	 */
	public String codigo;
	/**
	 * Cdigo del emisor.
	 * Su valor depende del formato:
	 * 		Formato 507: Emisor - sufijo
	 * 		Formato 502: Emisor
	 * 		Formato 508: Emisor - Entidad Tesorera
	 * 		Formato 521: Emisor
	 * 		Formato 522: Emisor
	 */
	public String emisor;
	/**
	 * Tipo del pago.
	 */
	public String tipo;
	/**
	 * Referencia del pago.
	 */
	public String referencia;
	/**
	 * Cada uno de los periodos de pago.
	 */
	public Map periodosPago;
	
	public DatosPago(){
	    ObjectUtils.initialize(this);
	}
	
	/**
	 * Convierte un Objeto DatosPago a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {   
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte un XML en un objeto DatosPago.
	 * @return Objeto DatosPago construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static DatosPago getObject(String xml) throws XOMarshallerException {    
	    return (DatosPago)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}
    
    /**
     * Mtodo que realiza la validacin de los datos de este objeto.
     * @return
     */
    public String validate() {
        
        String error_validacion = null;
        
        /**
         *  Se comprueba que el formato:
         *      - no es NULL.
         *      - que corresponde a uno de los formatos conocidos.
         */
        if (formato==null) {
            return ValidationConstants.FORMATO_NULL;            
        } 
        if(!formato.equals("507") && 
                !formato.equals("502") && 
                !formato.equals("508") && 
                !formato.equals("521")&& 
                !formato.equals("522")) {
            return ValidationConstants.FORMATO_DESCONOCIDO;            
        }        
        
        /** Se comprueba que el emisor:
         *      - no es NULL.
         *      - que es correcto en funcion de formato del pago. 
         */
        if (emisor==null) {
            return ValidationConstants.EMISOR_NULL;            
        }
        if (formato.equals("507")) {
            if (emisor.length()!=12) {
                return ValidationConstants.EMISOR_LONGITUD;            
            }            
            String entidad = emisor.substring(0,8);
            if (!NumberUtils.isNumber(entidad)) {
                return ValidationConstants.EMISOR_ENTIDAD;            
            } 
            String sufijo = emisor.substring(9,12);
            if (!NumberUtils.isNumber(sufijo)) {
                return ValidationConstants.EMISOR_SUFIJO;            
            }            
        } else {
            if (emisor.length()!=6) {
                return ValidationConstants.EMISOR_LONGITUD;            
            }            
            if (!NumberUtils.isNumber(emisor)) {
                return ValidationConstants.EMISOR_LONGITUD;            
            } 
        }            
        
        /** Se comprueba que el tipo:
         *      - no es NULL.
         *      - que es un numero de longitud 3.
         */
        if (tipo==null) {
            return ValidationConstants.TIPO_NULL;                       
        } 
        if (tipo.length()!=3) {
            return ValidationConstants.TIPO_LONGITUD;               
        } 
        if (!NumberUtils.isNumber(tipo)) {
            return ValidationConstants.TIPO;            
        } 
        
        /* Se comprueba que la referencia:
         *      - no es NULL.
         *      - que es un numero de la longitud adecuada al formato del pago.
         *      - que el codigo de control es correcto.
         */
        if (referencia==null) {
            return ValidationConstants.REFERENCIA_NULL;            
        } 
        if (!NumberUtils.isNumber(referencia)) {
            return ValidationConstants.REFERENCIA;            
        } 
        if(formato.equals("507")) {
            if(referencia.length()!=13) {
                return ValidationConstants.REFERENCIA_LONGITUD;            
            } 
        } else if(formato.equals("502") || formato.equals("508") || formato.equals("521")) {
            if(referencia.length()!=12) {
                return ValidationConstants.REFERENCIA_LONGITUD;            
            }
        } else if(formato.equals("522")) {
            if(referencia.length()!=14) {
                return ValidationConstants.REFERENCIA_LONGITUD;            
            }
        }   
        // Comprobamos en el fichero de configuracion si hay que validar el codigo de control
        // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        // Hay que coger de la configuracion el tipo de validacion del pago
        String validarCC = XMLProperties.get("p12f","configPasarela/validaciones/codigoControl");
        if(validarCC.equals("true")){
            // Se comprueban los codigos de control
            if(!ControlCodeHelper.compruebaCC(this)){
                return ValidationConstants.REFERENCIA_CONTROL;            
            }
        }
         
        /** Validar que los periodos de pago:
         *      - se corresponden con los periodos adecuados al formato del pago. 
         *      - contienen los datos adecuados al formato del pago.
         */
        if(!formato.equals("522")) {
            if(periodosPago==null || periodosPago.get(PeriodoPago.PERIODO_NORMAL)==null) {
                return ValidationConstants.PERIODOSPAGO;            
            }
            PeriodoPago periodoNormal = (PeriodoPago)periodosPago.get(PeriodoPago.PERIODO_NORMAL);
            error_validacion = periodoNormal.validate(formato);
            if(error_validacion!=null) {
                return error_validacion;
            }
        } else if(formato.equals("522")) {
            if(periodosPago==null || periodosPago.get(PeriodoPago.PERIODO_VOLUNTARIO)==null || periodosPago.get(PeriodoPago.PERIODO_CON_RECARGO)==null) {
                return ValidationConstants.PERIODOSPAGO;            
            }
            PeriodoPago periodoVoluntario = (PeriodoPago)periodosPago.get(PeriodoPago.PERIODO_VOLUNTARIO);
            error_validacion = periodoVoluntario.validate(formato);
            if(error_validacion!=null) {
                return error_validacion;
            }
            PeriodoPago periodoConRecargo = (PeriodoPago)periodosPago.get(PeriodoPago.PERIODO_CON_RECARGO);
            error_validacion = periodoConRecargo.validate(formato);
            if(error_validacion!=null) {
                return error_validacion;
            }
        }
        
        return error_validacion;
    }
}
